Package: origami
Title: Generalized Framework for Cross-Validation
Version: 1.0.8
Authors@R: c(
    person("Jeremy", "Coyle", email = "jeremyrcoyle@gmail.com",
           role = c("aut", "cph"),
           comment = c(ORCID = "0000-0002-9874-6649")),
    person("Nima", "Hejazi", email = "nh@nimahejazi.org",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7127-2789")),
    person("Ivana", "Malenica", email = "imalenica@berkeley.edu",
           role = "aut",
           comment = c(ORCID = "0000-0002-7404-8088")),
    person("Rachael", "Phillips", email = "rachaelvphillips@berkeley.edu",
           role = "aut",
           comment = c(ORCID = "0000-0002-8474-591X"))
  )
Maintainer: Nima Hejazi <nh@nimahejazi.org>
Description: A general framework for the application of cross-validation schemes
    to particular functions. By allowing arbitrary lists of results, origami
    accommodates a range of cross-validation applications. This implementation
    was first described by Coyle and Hejazi (2018) <doi:10.21105/joss.00512>.
Depends: R (>= 3.0.0),
License: GPL-3
URL: https://tlverse.org/origami/
BugReports: https://github.com/tlverse/origami/issues
Encoding: UTF-8
Imports: abind, methods, data.table, assertthat, future, future.apply,
        listenv
Suggests: testthat, class, rmarkdown, knitr, stringr, glmnet, forecast,
        randomForest
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-12 20:48:26 UTC; nih914
Author: Jeremy Coyle [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-9874-6649>),
  Nima Hejazi [aut, cre] (ORCID: <https://orcid.org/0000-0002-7127-2789>),
  Ivana Malenica [aut] (ORCID: <https://orcid.org/0000-0002-7404-8088>),
  Rachael Phillips [aut] (ORCID: <https://orcid.org/0000-0002-8474-591X>)
Repository: CRAN
Date/Publication: 2026-05-18 05:50:11 UTC
Built: R 4.5.2; ; 2026-05-18 08:47:17 UTC; unix
