# Generated by roxygen2: do not edit by hand

export(combine_results)
export(combiner_array)
export(combiner_c)
export(combiner_factor)
export(combiner_rbind)
export(cross_validate)
export(fold_from_foldvec)
export(fold_index)
export(folds2foldvec)
export(folds_bootstrap)
export(folds_loo)
export(folds_montecarlo)
export(folds_resubstitution)
export(folds_rolling_origin)
export(folds_rolling_origin_pooled)
export(folds_rolling_window)
export(folds_rolling_window_pooled)
export(folds_vfold)
export(folds_vfold_rolling_origin_pooled)
export(folds_vfold_rolling_window_pooled)
export(id_folds_to_folds)
export(make_fold)
export(make_folds)
export(make_repeated_folds)
export(training)
export(validation)
export(wrap_in_try)
import(methods)
importFrom(abind,abind)
importFrom(assertthat,assert_that)
importFrom(data.table,rbindlist)
importFrom(future,future)
importFrom(future,multicore)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(future.apply,future_lapply)
importFrom(listenv,listenv)
